/* file:	src/osgVRPN/AnalogManipulator.cpp
 * author:	Mike Weiblen mew@mew.cx 2003-12-27
 * copyright:	(C) 2003 Michael Weiblen
 * license:	OpenSceneGraph Public License (OSGPL)
*/

// #include <cassert>
#include <osg/Notify>
#include <osgVRPN/AnalogManipulator.h>

using namespace osgVRPN;

///////////////////////////////////////////////////////////////////////////

AnalogManipulator::AnalogManipulator( osg::ref_ptr<Analog> analog )
{
    setAnalog( analog );
}

AnalogManipulator::AnalogManipulator( const char* analogName )
{
    setAnalog( new Analog( analogName ) );
}

///////////////////////////////////////////////////////////////////////////

void AnalogManipulator::setByMatrix( const osg::Matrixd& matrix )
{
    // TODO how does OSG's use fit with this implementation?
    _origin = matrix;
}

void AnalogManipulator::setByInverseMatrix( const osg::Matrixd& matrix )
{
    // TODO how does OSG's use fit with this implementation?
    setByMatrix( osg::Matrixd::inverse(matrix) );
}

osg::Matrixd AnalogManipulator::getMatrix() const
{
    osg::Matrixd matrix;

    if( _analog.valid() )
	matrix = _analog->getMatrix() * _origin;
    else
	matrix = _origin;

    return matrix;
}

osg::Matrixd AnalogManipulator::getInverseMatrix() const
{
    osg::Matrixd matrix;

    if( _analog.valid() )
	matrix = osg::Matrixd::inverse(_origin) * _analog->getInverseMatrix();
    else
	matrix = osg::Matrixd::inverse(_origin);

    return matrix;
}

///////////////////////////////////////////////////////////////////////////

void AnalogManipulator::setNode(osg::Node* node)
{
    _node = node;
    if( _node.get() )
    {
        const osg::BoundingSphere& boundingSphere=_node->getBound();

	setByMatrix( osg::Matrix::translate( 0, 0, 2*boundingSphere._radius ) *
		osg::Matrixd::rotate( 0.9, 1, 0, 0 ) );
    }
}

///////////////////////////////////////////////////////////////////////////
// Handle "input events".
// VRPN traffic doesn't cause OSG input events, so we depend on the
// GUIEventAdapter::FRAME event to ensure we get called often enough to
// process VRPN messages.

bool AnalogManipulator::handle(
	const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& )
{
    switch( ea.getEventType() )
    {
	case osgGA::GUIEventAdapter::FRAME:
	{
	    if( _analog.valid() )
		_analog->update();
	    return true;
	}

	default:
	    break;
    }

    return false;
}

/*EOF*/
